==> portmanager-0.3.4.tar.gz: portmanager-0.3.4/libMGPM/src/MGPMrMissingDependencies.c <==
/*
	Copyright (C) 2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/

#include	<libMGPM.h>

int	MGPMrMissingDependencies( structProperty* property )
{
	FILE*	makeDependAllStream;
	char	exact[]					= "exact";
	char	id[]					= "MGPMrMissingDependencies";
	char*	buffer					= NULL;
	char*	command					= NULL;
	char*	dependencyDbDependencyPortDirPtr	= NULL;
	char*	dependencyDbDependencyPortNamePtr	= NULL;
	char*	dependencyDbPortDirPtr			= NULL;
	char*	dependencyDbPortNamePtr			= NULL;
	char*	installedPortDirPtr			= NULL;
	char*	optionsPtr				= NULL;
	int	IDX_installedPortsDb			= 1;
	int	bufferSize				= 0xffff;
	int	dependencyPortsDbIDX			= 1;
	int	dependencyPortsDbQTY			= 0;
	int	idx1					= 0;
	int	idx2					= 0;
	int	isOk					= 0;

	command				= (char*)calloc( 255, 1 );
	buffer				= (char*)calloc( bufferSize,1 );

	property->configDb		= MGdbOpen( property->configDbFileName );
	property->dependencyPortsDb	= MGdbOpen( property->dependencyPortsDbFileName );
	property->installedPortsDb	= MGdbOpen( property->installedPortsDbFileName );
 	property->oldPortsDb		= MGdbOpen( property->oldPortsDbFileName );
 
	fprintf( stdout,"%s\n", SINGLE_LINES );
	fprintf( stdout,"%s %s info: looking for missing dependent ports\n", id, PACKAGE_VERSION );
	fprintf( stdout,"%s\n", SINGLE_LINES );
	while( fflush( stdout ) );
	
	/*****************************************************************************/
	/* insure all dependency ports are installed, if not check "make describe"   */
	/* to veryify acuracy of the dependency *NEW ver 0.2.3* If the dependency    */
	/* directory is not listed by make describe then do not install it!!!        */
	/*****************************************************************************/

	dependencyPortsDbQTY	= MGdbGetRecordQty( property->dependencyPortsDb );
	IDX_installedPortsDb	= 1;	/* start at one to skip over field names */
	dependencyPortsDbIDX	= 1;

	/* cycle through each dependency port and see if it is installed */
	while( dependencyPortsDbIDX <  dependencyPortsDbQTY )
	{
		dependencyDbDependencyPortDirPtr	= MGdbGet( property->dependencyPortsDb, dependencyPortsDbIDX,
								property->fieldDependencyDbDependencyPortDir );
		/*
		 * see if dependencyPortsDb's dependencyDbDependencyPortNamePtr is installed according to installedPortsDb
		 */ 
		MGdbGoTop( property->installedPortsDb );
		installedPortDirPtr	= MGdbSeek( property->installedPortsDb, property->fieldInstalledPortsDbPortDir, dependencyDbDependencyPortDirPtr,
						property->fieldInstalledPortsDbPortDir, exact );
		if( installedPortDirPtr )
		{
			/* if here then dpendency port is installed */
			dependencyPortsDbIDX++;
			continue;
		}

		/* 
		if here then dependency port is MISSING, need to verify it is still a valid dependency of the port
		named in dependencyPortsDb's fieldDependencyDbPortName. If it is valid then add to oldPortsDb
		*/
		dependencyDbDependencyPortNamePtr	= MGdbGet( property->dependencyPortsDb, dependencyPortsDbIDX,
								property->fieldDependencyDbDependencyPortName );

		dependencyDbPortNamePtr		= MGdbGet( property->dependencyPortsDb, dependencyPortsDbIDX, property->fieldDependencyDbPortName );
						
		MGdbGoTop( property->installedPortsDb );
		dependencyDbPortDirPtr		= MGdbSeek( property->installedPortsDb, property->fieldInstalledPortsDbPortName,
							dependencyDbPortNamePtr, property->fieldInstalledPortsDbPortDir, exact );

		fprintf( stdout, "%s\n", SINGLE_LINES );
		fprintf( stdout, "-=>MISSING<=- %s[%s] may be a dependency of %s\n", dependencyDbDependencyPortNamePtr,dependencyDbDependencyPortDirPtr,
			dependencyDbPortNamePtr );

		/*
		 * set "isOk" if make all-depends-list of child port contains dependencyPortDirPtr
		 */
		MGdbGoTop( property->configDb );
		optionsPtr = MGdbSeek( property->configDb, property->fieldConfigDbKey, dependencyDbPortDirPtr, property->fieldConfigDbValue, exact );

		strcpy( command, "cd " );
		strcat( command, PORTSDIR );
		strcat( command, dependencyDbPortDirPtr );
		strcat( command,  "; " );
		strcat( command, "make -V RUN_DEPENDS -V LIB_DEPENDS -V DEPENDS" );
		if( optionsPtr )
		{
			strcat( command, optionsPtr );
		}
		fprintf( stdout, "verifying dependency status of %s %s by executing command:\n%s\n",
					dependencyDbDependencyPortNamePtr, dependencyDbDependencyPortDirPtr, command );
		while( fflush( stdout ) );

		makeDependAllStream	= popen( command, "r" );
		buffer[0]	= 0;
		fread( buffer, bufferSize-1, 1, makeDependAllStream );

		idx1	= 0;
		while( idx1 < bufferSize )
		{
			if( buffer[idx1] == ':' )
			{
				idx1++;
				idx2	= idx1;
				while( idx2 < bufferSize && buffer[idx2] != ' ' && buffer[idx2] != LINEFEED && buffer[idx2] != TAB &&
						buffer[idx2] != ':' )
				{
					idx2++;
				}
				buffer[idx2]		= 0;
				if( idx1 + 11 < bufferSize )
				{
					isOk	= strcmp( dependencyDbDependencyPortDirPtr, buffer + idx1 + 10 );
				}
				else
				{
					assert(0);
				}
			}
			idx1++;
		}

		/*
		 * set "isOk" if make all-depends-list of child port contains dependencyPortDirPtr
		 */

		pclose( makeDependAllStream );

		if( isOk == 0 )
		{
			fprintf( stdout, "  *  *  *  *\n" );
			fprintf( stdout, "%s is indeed a missing dependency, adding to list of things to be updated\n",
				dependencyDbDependencyPortNamePtr );
			/*
			 * only add if not allready in oldPortsDb
			 */
			MGdbGoTop( property->oldPortsDb );
			if( MGdbSeek( property->oldPortsDb, property->fieldOldPortsDbOldPortDir, dependencyDbDependencyPortDirPtr,
				property->fieldOldPortsDbOldPortDir, exact ) == 0 )
			{
				MGdbAdd( property->oldPortsDb, dependencyDbDependencyPortDirPtr, dependencyDbDependencyPortNamePtr,
					"MISSING is a dependency of", dependencyDbPortNamePtr, dependencyDbPortDirPtr, NULL );
			}
		}
		else	/* from if(isOk) */
		{
			fprintf( stdout, 
				"%s is no longer a dependency of %s\n forcing rebuild of %s to fix /var/db/pkg/%s/+CONTENTS file.\n",
				dependencyDbDependencyPortNamePtr, dependencyDbPortNamePtr, dependencyDbPortNamePtr, dependencyDbPortDirPtr );
			while( fflush( stdout ) );
			MGdbAdd( property->oldPortsDb, dependencyDbPortDirPtr, dependencyDbPortNamePtr,
				"built with old/no longer in service dependency of", dependencyDbDependencyPortNamePtr,
				dependencyDbDependencyPortDirPtr, NULL );
		}
		dependencyPortsDbIDX++;
	}
	MGdbDestroy( property->configDb );
	MGdbDestroy( property->dependencyPortsDb );
	MGdbDestroy( property->installedPortsDb );
	MGdbDestroy( property->oldPortsDb );
	free( buffer );
	free( command );
	return( 0 );
}

==> portmanager-0.4.1.tar.gz: portmanager-0.4.1/libMGPM/src/MGPMrMissingDependencies.c <==
/*
	Copyright (C) 2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/

#include	<libMGPM.h>

int	MGPMrMissingDependencies( structProperty* property )
{
	FILE*	makeDependAllStream;
	char	exact[]					= "exact";
	char	id[]					= "MGPMrMissingDependencies";
	char*	buffer					= NULL;
	char*	command					= NULL;
	char*	dependencyDbDependencyPortDirPtr	= NULL;
	char*	dependencyDbDependencyPortNamePtr	= NULL;
	char*	dependencyDbPortDirPtr			= NULL;
	char*	dependencyDbPortNamePtr			= NULL;
	char*	installedPortDirPtr			= NULL;
	char*	optionsPtr				= NULL;
	int	IDX_installedPortsDb			= 1;
	int	bufferSize				= 0xffff;
	int	dependencyPortsDbIDX			= 1;
	int	dependencyPortsDbQTY			= 0;
	int	idx1					= 0;
	int	idx2					= 0;
	int	isOk					= 0;

	command				= (char*)calloc( 255, 1 );
	buffer				= (char*)calloc( bufferSize,1 );

	property->configDb		= MGdbOpen( property->configDbFileName );
	property->dependencyPortsDb	= MGdbOpen( property->dependencyPortsDbFileName );
	property->installedPortsDb	= MGdbOpen( property->installedPortsDbFileName );
 	property->oldPortsDb		= MGdbOpen( property->oldPortsDbFileName );
 
	fprintf( stdout,"%s\n", SINGLE_LINES );
	fprintf( stdout,"%s %s info: looking for missing dependent ports\n", id, PACKAGE_VERSION );
	fprintf( stdout,"%s\n", SINGLE_LINES );
	while( fflush( stdout ) );
	
	/*****************************************************************************/
	/* insure all dependency ports are installed, if not check "make describe"   */
	/* to veryify acuracy of the dependency *NEW ver 0.2.3* If the dependency    */
	/* directory is not listed by make describe then do not install it!!!        */
	/*****************************************************************************/

	dependencyPortsDbQTY	= MGdbGetRecordQty( property->dependencyPortsDb );
	IDX_installedPortsDb	= 1;	/* start at one to skip over field names */
	dependencyPortsDbIDX	= 1;

	/* cycle through each dependency port and see if it is installed */
	while( dependencyPortsDbIDX <  dependencyPortsDbQTY )
	{
		dependencyDbDependencyPortDirPtr	= MGdbGet( property->dependencyPortsDb, dependencyPortsDbIDX,
								property->fieldDependencyDbDependencyPortDir );
		/*
		 * see if dependencyPortsDb's dependencyDbDependencyPortNamePtr is installed according to installedPortsDb
		 */ 
		MGdbGoTop( property->installedPortsDb );
		installedPortDirPtr	= MGdbSeek( property->installedPortsDb, property->fieldInstalledPortsDbPortDir, dependencyDbDependencyPortDirPtr,
						property->fieldInstalledPortsDbPortDir, exact );
		if( installedPortDirPtr )
		{
			/* if here then dpendency port is installed */
			dependencyPortsDbIDX++;
			continue;
		}

		/* 
		if here then dependency port is MISSING, need to verify it is still a valid dependency of the port
		named in dependencyPortsDb's fieldDependencyDbPortName. If it is valid then add to oldPortsDb
		*/
		dependencyDbDependencyPortNamePtr	= MGdbGet( property->dependencyPortsDb, dependencyPortsDbIDX,
								property->fieldDependencyDbDependencyPortName );

		dependencyDbPortNamePtr		= MGdbGet( property->dependencyPortsDb, dependencyPortsDbIDX, property->fieldDependencyDbPortName );
						
		MGdbGoTop( property->installedPortsDb );
		dependencyDbPortDirPtr		= MGdbSeek( property->installedPortsDb, property->fieldInstalledPortsDbPortName,
							dependencyDbPortNamePtr, property->fieldInstalledPortsDbPortDir, exact );

		fprintf( stdout, "%s\n", SINGLE_LINES );
		fprintf( stdout, "-=>MISSING<=- %s[%s] may be a dependency of %s\n", dependencyDbDependencyPortNamePtr,dependencyDbDependencyPortDirPtr,
			dependencyDbPortNamePtr );

		/*
		 * set "isOk" if make all-depends-list of child port contains dependencyPortDirPtr
		 */
		MGdbGoTop( property->configDb );
		optionsPtr = MGdbSeek( property->configDb, property->fieldConfigDbKey, dependencyDbPortDirPtr, property->fieldConfigDbValue, exact );

		MGmStrcpy( command, "cd " );
		MGmStrcat( command, PORTSDIR );
		MGmStrcat( command, dependencyDbPortDirPtr );
		MGmStrcat( command,  "; " );
		MGmStrcat( command, "make -V RUN_DEPENDS -V LIB_DEPENDS -V DEPENDS" );
		if( optionsPtr )
		{
			MGmStrcat( command, optionsPtr );
		}
		fprintf( stdout, "verifying dependency status of %s %s by executing command:\n%s\n",
					dependencyDbDependencyPortNamePtr, dependencyDbDependencyPortDirPtr, command );
		while( fflush( stdout ) );

		makeDependAllStream	= popen( command, "r" );
		buffer[0]	= 0;
		fread( buffer, bufferSize-1, 1, makeDependAllStream );

		idx1	= 0;
		while( idx1 < bufferSize )
		{
			if( buffer[idx1] == ':' )
			{
				idx1++;
				idx2	= idx1;
				while( idx2 < bufferSize && buffer[idx2] != ' ' && buffer[idx2] != LINEFEED && buffer[idx2] != TAB &&
						buffer[idx2] != ':' )
				{
					idx2++;
				}
				buffer[idx2]		= 0;
				if( idx1 + 11 < bufferSize )
				{
					isOk	= strcmp( dependencyDbDependencyPortDirPtr, buffer + idx1 + 10 );
				}
				else
				{
					assert(0);
				}
			}
			idx1++;
		}

		/*
		 * set "isOk" if make all-depends-list of child port contains dependencyPortDirPtr
		 */

		pclose( makeDependAllStream );

		if( isOk == 0 )
		{
			fprintf( stdout, "  *  *  *  *\n" );
			fprintf( stdout, "%s is indeed a missing dependency, adding to list of things to be updated\n",
				dependencyDbDependencyPortNamePtr );
			/*
			 * only add if not allready in oldPortsDb
			 */
			MGdbGoTop( property->oldPortsDb );
			if( MGdbSeek( property->oldPortsDb, property->fieldOldPortsDbOldPortDir, dependencyDbDependencyPortDirPtr,
				property->fieldOldPortsDbOldPortDir, exact ) == 0 )
			{
				MGdbAdd( property->oldPortsDb, dependencyDbDependencyPortDirPtr, dependencyDbDependencyPortNamePtr,
					"MISSING is a dependency of", dependencyDbPortNamePtr, dependencyDbPortDirPtr, NULL );
			}
		}
		else	/* from if(isOk) */
		{
			fprintf( stdout, 
				"%s is no longer a dependency of %s\n forcing rebuild of %s to fix /var/db/pkg/%s/+CONTENTS file.\n",
				dependencyDbDependencyPortNamePtr, dependencyDbPortNamePtr, dependencyDbPortNamePtr, dependencyDbPortDirPtr );
			while( fflush( stdout ) );
			MGdbAdd( property->oldPortsDb, dependencyDbPortDirPtr, dependencyDbPortNamePtr,
				"built with old/no longer in service dependency of", dependencyDbDependencyPortNamePtr,
				dependencyDbDependencyPortDirPtr, NULL );
		}
		dependencyPortsDbIDX++;
	}
	MGdbDestroy( property->configDb );
	MGdbDestroy( property->dependencyPortsDb );
	MGdbDestroy( property->installedPortsDb );
	MGdbDestroy( property->oldPortsDb );
	free( buffer );
	free( command );
	return( 0 );
}
